@@ -25,18 +25,30 @@ test_error(mirai(), "missing expression, perhaps wrap in {}?")
 test_error(mirai(a, 1), "all `...` arguments must be named")
 test_error(mirai(a, .args = list(1)), "all items in `.args` must be named")
 test_error(mirai_map(1:2, identity))
-test_error(daemons(url = "URL"), "Invalid argument")
+test_error(daemons(url = "URL"))
 test_error(daemons(-1), "zero or greater")
 test_error(daemons(raw(0L)), "must be numeric")
 test_error(daemons(1, dispatcher = ""))
 test_error(daemons(url = local_url(), dispatcher = NA))
-test_error(daemon("URL"), "Invalid argument")
+test_error(daemon("URL"))
 test_error(launch_local(1L), "daemons must be set")
 test_error(race_mirai(list()), "daemons must be set")
 test_false(daemons_set())
 test_error(require_daemons())
+if (mirai:::cli_enabled) {
+  ns <- getNamespace("mirai")
+  unlockBinding("cli_enabled", ns)
+  ns[["cli_enabled"]] <- FALSE
+  cli_disabled <- TRUE
+} else {
+  cli_disabled <- FALSE
+}
 test_error(mirai:::stop_d(NULL), "No daemons set.")
 test_error(mirai:::stop_d("default"), "No daemons set for the 'default' compute profile.")
+if (cli_disabled) {
+  ns[["cli_enabled"]] <- TRUE
+  lockBinding("cli_enabled", ns)
+}
 test_type("character", host_url())
 test_type("character", names(host_url()))
 test_true(all(startsWith(host_url(tls = TRUE), "tls")))
@@ -86,7 +98,6 @@ connection && {
   test_true(d <- daemons(1L, dispatcher = FALSE, asyncdial = FALSE, seed = 1546L))
   test_print(d)
   test_true(daemons_set())
-  test_true(suppressWarnings(require_daemons(parent.frame())))
   me <- mirai(mirai::mirai(), .timeout = 2000L)[]
   test_notnull(me)
   if (is_mirai_error(me)) test_type("list", me$stack.trace)
@@ -161,7 +172,8 @@ connection && {
 connection && {
   Sys.sleep(1L)
   m <- with(daemons(1, dispatcher = FALSE, .compute = "ml"), {
-    if (is.null(tryCatch(mirai_map(list(1, "a", 2), sum)[.stop], error = function(e) NULL)))
+    if (tryCatch(mirai_map(list(1, "a", 2), sum)[.stop], error = function(e) TRUE) &&
+        tryCatch(mirai_map(list(1, "a", 2), sum)[.flat], error = function(e) TRUE))
       mirai_map(1:3, rnorm, .args = list(mean = 20, 2))[]
   })
   test_false(is_mirai_map(m))
@@ -358,25 +370,26 @@ connection && Sys.getenv("NOT_CRAN") == "true" && {
   test_equal(mirai(1)[], mirai(1)[])
   m <- mirai(0L)
   Sys.sleep(1L)
-  res <- status()
-  test_zero(res$connections)
-  test_equal(res$mirai[["awaiting"]], 1L)
+  res <- info()
+  test_zero(res[["connections"]])
+  test_equal(res[["awaiting"]], 1L)
   test_equal(launch_local(1, idletime = 5000L, walltime = 500L), 1L)
   test_zero(m[])
   Sys.sleep(1L)
-  res <- status()
-  test_zero(res$connections)
-  test_equal(info()[["cumulative"]], 2L)
+  res <- info()
+  test_zero(res[["connections"]])
+  test_equal(res[["cumulative"]], 2L)
   test_false(daemons(0))
   test_true(daemons(1, dispatcher = FALSE, maxtasks = 1L))
   test_zero(mirai(0L)[])
   Sys.sleep(0.5)
-  test_zero(status()$connections)
+  test_zero(info()[["connections"]])
   test_equal(launch_local(1, idletime = 200L, walltime = 1000L), 1L)
   test_zero(mirai(0)[])
   Sys.sleep(1L)
-  test_zero(status()$connections)
-  test_true(is.na(info()[["cumulative"]]))
+  res <- info()
+  test_zero(res[["connections"]])
+  test_true(is.na(res[["cumulative"]]))
   test_false(daemons(0))
 }
 # mirai cancellation tests
@@ -389,16 +402,16 @@ connection && Sys.getenv("NOT_CRAN") == "true" && {
   Sys.sleep(0.1)
   test_true(stop_mirai(m2))
   test_true(stop_mirai(m1))
-  test_equal(m2$data, 20L)
-  test_equal(m1$data, 20L)
+  test_true(m2$data >= 20L)
+  test_true(m1$data >= 20L)
   test_class("errorValue", mirai(res)[])
   m <- mirai_map(1:10, function(x) { Sys.sleep(2); y <<- TRUE })
   Sys.sleep(0.1)
   s <- stop_mirai(m)
-  test_equal(sum(unlist(m[])), 200L)
+  test_true(all(m[] >= 20L))
   test_class("errorValue", mirai(y)[])
   test_identical(s, !logical(10L))
-  test_equal(status()$connections, 1L)
+  test_equal(info()[["connections"]], 1L)
   test_equal(length(nextget("url")), 1L)
   test_class("miraiLaunchCmd", launch_remote(1))
   test_false(daemons(0))
@@ -413,13 +426,13 @@ connection && Sys.getenv("NOT_CRAN") == "true" && {
   test_class("mirai_map", everywhere(a <<- FALSE))
   test_true(all(everywhere(a <<- TRUE)[.flat]))
   test_true(all(mirai_map(1:10, function(x) a)[.flat]))
-  for (i in seq_len(10000L)) {q[[i]] <- mirai(1L); attr(q[[i]], "status") <- status()}
+  for (i in seq_len(10000L)) {q[[i]] <- mirai(1L); attr(q[[i]], "info") <- info()}
   test_equal(sum(unlist(collect_mirai(q))), 10000L)
-  test_true(all(as.logical(lapply(lapply(q, attr, "status"), is.list))))
-  for (i in seq_len(10000L)) {q[[i]] <- mirai({Sys.sleep(0.001); rnorm(1)}); attr(q[[i]], "status") <- status()}
+  test_true(all(as.logical(lapply(lapply(q, attr, "info"), is.integer))))
+  for (i in seq_len(10000L)) {q[[i]] <- mirai({Sys.sleep(0.001); rnorm(1)}); attr(q[[i]], "info") <- info()}
   test_equal(length(unique(unlist(collect_mirai(q)))), 10000L)
-  test_true(all(as.logical(lapply(lapply(q, attr, "status"), is.list))))
-  test_equal(status()[["mirai"]][["completed"]], 20020L)
+  test_true(all(as.logical(lapply(lapply(q, attr, "info"), is.integer))))
+  test_equal(info()[["completed"]], 20020L)
   test_false(daemons(0))
 }
 # reproducible RNG tests
@@ -440,73 +453,70 @@ connection && Sys.getenv("NOT_CRAN") == "true" && {
   test_false(daemons_set("gpu"))
   test_identical(m, n)
 }
-connection && requireNamespace("otel", quietly = TRUE) && Sys.getenv("NOT_CRAN") == "true" && {
-  record <- tryCatch(
-    getNamespace("otelsdk")$with_otel_record({
-      ns <- getNamespace("mirai")
-      unlockBinding("otel_tracing", ns)
-      ns[["otel_tracing"]] <- TRUE
-      lockBinding("otel_tracing", ns)
-      unlockBinding("otel_tracer", ns)
-      ns[["otel_tracer"]] <- otel::get_tracer(name = mirai:::otel_tracer_name)
-      lockBinding("otel_tracer", ns)
-      url <- local_url()
-      test_true(daemons(url = url, dispatcher = FALSE))
-      mp <- mirai_map(1:3, rnorm)
-      m1 <- mirai(stop("error"))
-      m2 <- mirai(getNamespace("rlang")$interrupt())
-      test_equal(daemon(url = url, maxtasks = 5L, dispatcher = FALSE), 3L)
-      test_true(is_error_value(m2[]))
-      test_false(daemons(0))
-    }),
-    error = function(cnd) NULL
-  )
-  length(record) && {
-    traces <- record$traces
-    test_equal(length(traces), 15L)
-    test_equal(traces[[1L]]$name, "daemons")
-    test_equal(traces[[1L]]$kind, "internal")
-    test_equal(traces[[1L]]$attributes$url, url)
-    test_equal(traces[[1L]]$attributes$n, 1L)
-    test_equal(traces[[1L]]$attributes$dispatcher, "false")
-    test_equal(traces[[1L]]$attributes$compute_profile, "default")
-    test_equal(traces[[2L]]$name, "mirai")
-    test_equal(traces[[2L]]$kind, "client")
-    test_equal(traces[[2L]]$links[[1L]]$span_id, traces[[1L]]$span_id)
-    test_true(traces[[3L]]$attributes$mirai.id > traces[[2L]]$attributes$mirai.id)
-    test_equal(traces[[3L]]$parent, traces[[2L]]$parent)
-    test_equal(traces[[4L]]$links[[1L]]$span_id, traces[[3L]]$links[[1L]]$span_id)
-    test_equal(traces[[5L]]$name, "mirai_map")
-    test_equal(traces[[5L]]$kind, "internal")
-    test_equal(traces[[5L]]$links[[1L]]$span_id, traces[[1L]]$span_id)
-    test_equal(traces[[5L]]$span_id, traces[[3L]]$parent)
-    test_equal(traces[[5L]]$links[[1L]]$span_id, traces[[1L]]$span_id)
-    test_equal(traces[[6L]]$name, "mirai")
-    test_equal(traces[[6L]]$kind, "client")
-    test_equal(traces[[6L]]$parent, record$traces[[5L]]$parent)
-    test_equal(traces[[6L]]$links[[1L]]$span_id, traces[[1L]]$span_id)
-    test_equal(traces[[8L]]$name, "daemon")
-    test_equal(traces[[8L]]$kind, "internal")
-    test_equal(traces[[8L]]$attributes$url, url)
-    test_equal(traces[[9L]]$name, "daemon->eval")
-    test_equal(traces[[9L]]$kind, "server")
-    test_equal(traces[[9L]]$parent, traces[[2L]]$span_id)
-    test_equal(traces[[9L]]$links[[1L]]$span_id, traces[[8L]]$span_id)
-    test_equal(traces[[12L]]$status, "error")
-    test_equal(traces[[12L]]$description, "miraiError")
-    test_equal(traces[[13L]]$status, "error")
-    test_equal(traces[[13L]]$description, "miraiInterrupt")
-    test_equal(traces[[14L]]$name, "daemon->end")
-    test_equal(traces[[14L]]$kind, "internal")
-    test_equal(traces[[14L]]$attributes$url, url)
-    test_equal(traces[[14L]]$links[[1L]]$span_id, traces[[8L]]$span_id)
-    test_equal(traces[[15L]]$name, "daemons->reset")
-    test_equal(traces[[15L]]$kind, "internal")
-    test_equal(traces[[15L]]$attributes$url, url)
-    test_equal(traces[[15L]]$attributes$n, 1L)
-    test_equal(traces[[15L]]$attributes$dispatcher, "false")
-    test_equal(traces[[15L]]$attributes$compute_profile, "default")
-  }
+connection && requireNamespace("otelsdk", quietly = TRUE) && Sys.getenv("NOT_CRAN") == "true" && {
+  record <- otelsdk::with_otel_record({
+    mirai:::otel_refresh_tracer("mirai")
+    url <- local_url()
+    purl <- nanonext::parse_url(url)
+    test_true(daemons(url = url, dispatcher = FALSE))
+    mp <- mirai_map(1:3, rnorm)
+    m1 <- mirai(stop("error"))
+    m2 <- mirai(getNamespace("rlang")$interrupt())
+    test_equal(daemon(url = url, autoexit = FALSE, maxtasks = 5L, dispatcher = FALSE), 3L)
+    test_true(is_error_value(m2[]))
+    test_false(daemons(0))
+  })
+  mirai:::otel_refresh_tracer("mirai")
+  traces <- record$traces
+  test_equal(length(traces), 15L)
+  test_true(startsWith(traces[[1L]]$name, "daemons set"))
+  test_equal(traces[[1L]]$kind, "internal")
+  test_equal(traces[[1L]]$attributes$server.address, purl[["path"]])
+  test_equal(length(traces[[1L]]$attributes$server.port), 0L)
+  test_equal(traces[[1L]]$attributes$network.transport, purl[["scheme"]])
+  test_false(traces[[1L]]$attributes$mirai.dispatcher)
+  test_equal(traces[[1L]]$attributes$mirai.compute, "default")
+  test_equal(traces[[2L]]$name, "mirai")
+  test_equal(traces[[2L]]$kind, "client")
+  test_equal(traces[[2L]]$links[[1L]]$span_id, traces[[1L]]$span_id)
+  test_true(traces[[3L]]$attributes$mirai.id > traces[[2L]]$attributes$mirai.id)
+  test_equal(traces[[3L]]$parent, traces[[2L]]$parent)
+  test_equal(traces[[4L]]$links[[1L]]$span_id, traces[[3L]]$links[[1L]]$span_id)
+  test_equal(traces[[5L]]$name, "mirai_map")
+  test_equal(traces[[5L]]$kind, "internal")
+  test_equal(traces[[5L]]$links[[1L]]$span_id, traces[[1L]]$span_id)
+  test_equal(traces[[5L]]$span_id, traces[[3L]]$parent)
+  test_equal(traces[[5L]]$links[[1L]]$span_id, traces[[1L]]$span_id)
+  test_equal(traces[[6L]]$name, "mirai")
+  test_equal(traces[[6L]]$kind, "client")
+  test_equal(traces[[6L]]$parent, record$traces[[5L]]$parent)
+  test_equal(traces[[6L]]$links[[1L]]$span_id, traces[[1L]]$span_id)
+  test_true(startsWith(traces[[8L]]$name, "daemon connect"))
+  test_equal(traces[[8L]]$kind, "internal")
+  test_equal(traces[[8L]]$attributes$server.address, purl[["path"]])
+  test_equal(length(traces[[8L]]$attributes$server.port), 0L)
+  test_equal(traces[[8L]]$attributes$network.transport, purl[["scheme"]])
+  test_equal(traces[[9L]]$name, "daemon eval")
+  test_equal(traces[[9L]]$kind, "server")
+  test_equal(traces[[9L]]$parent, traces[[2L]]$span_id)
+  test_equal(traces[[9L]]$links[[1L]]$span_id, traces[[8L]]$span_id)
+  test_equal(traces[[12L]]$status, "error")
+  test_equal(traces[[12L]]$description, "miraiError")
+  test_equal(traces[[13L]]$status, "error")
+  test_true(startsWith(traces[[13L]]$description, "mirai"))
+  test_true(startsWith(traces[[14L]]$name, "daemon disconnect"))
+  test_equal(traces[[14L]]$kind, "internal")
+  test_equal(traces[[14L]]$attributes$server.address, purl[["path"]])
+  test_equal(length(traces[[14L]]$attributes$server.port), 0L)
+  test_equal(traces[[14L]]$attributes$network.transport, purl[["scheme"]])
+  test_equal(traces[[14L]]$links[[1L]]$span_id, traces[[8L]]$span_id)
+  test_true(startsWith(traces[[15L]]$name, "daemons reset"))
+  test_equal(traces[[15L]]$kind, "internal")
+  test_equal(traces[[15L]]$attributes$server.address, purl[["path"]])
+  test_equal(length(traces[[15L]]$attributes$server.port), 0L)
+  test_equal(traces[[15L]]$attributes$network.transport, purl[["scheme"]])
+  test_false(traces[[15L]]$attributes$mirai.dispatcher)
+  test_equal(traces[[15L]]$attributes$mirai.compute, "default")
 }
 test_false(daemons(0))
 Sys.sleep(1L)
